/***********************************************************************

     IBM Confidential

     OCO Source Materials

     (C) Copyright IBM Corporation 1995-96

     The source code for this program is not published or
     otherwise divested of its trade secrets, irrespective of
     what has been deposited with the U.S. Copyright Office.

***********************************************************************/

#ifndef _PROXY_BINDINGS_H_
#define _PROXY_BINDINGS_H_

#undef SSL_SUPPORT 
#include "ProxyBasicIncludes.h"
#include "ProxyTable.h"

class ProxyHandle;

#define BINDINGS_VERSION  "Binding_Vers"


class Information {
protected:
  ProxyHandle *proxy;

 private:
  HashTable *hash;

  void Plain2HTMLString(char* in, char* out);
    
  public:
  Information();
  ~Information();

  virtual Information* clone();
  void UseProxy(char* address, int port=1998);
  void SetTimeoutForMgmtOperations(unsigned short _timeout);

  void SetAttribute(char* name, char* value, int copyValue=1);
  char* GetAttribute(char* name);
  
  void RemoveAttribute(char* name);
  char** GetAttributes();
  char** GetAttributeKeys();
  void RemoveAllAttributes();
  void CopyValue(Information* in);
  char* Marshall();
  void print();
  int GetProxyBindingsVersion();
};

/**********************************************************************************/

class ProxyHandle {

 private:
  int sockId, port;
  unsigned short timeout;
  char *host, errMsg[512], *buffer;

  void SendStringWithCr(char* s);	
  char* ReceiveStringWithCr();
  void SkipHTTPResponseHeader();
  void SendHTTPRequest(char* url);

public:
  ProxyHandle(char* host=NULL /* Local */, int port=1998) /* throw(char*) */;	
  ~ProxyHandle();
  
  Information** SendRequest(char* operation, char* context, Information* input);	
  char* SendOffLineRequest(char* operation, char* context, char* input);
  Information* SendOffLineReqWithMultipleOut(char* operation, char* context, char* input);  
  void SetTimeoutForMgmtOperations(unsigned short _timeout) { timeout=_timeout; }
  unsigned int GetTimeoutForMgmtOperations() { return(timeout); }
  int GetProxyBindingsVersion();

  int GetRemoteHostPort()   { return(port); }
  char* GetRemoteHostHost() { return(host); }
};

/**********************************************************************************/

#define CMIP_GET     "CMIP_Get"
#define CMIP_SET     "CMIP_Set"
#define CMIP_DELETE  "CMIP_Delete"
#define CMIP_CREATE  "CMIP_Create"
#define CMIP_ACTION  "CMIP_Action"
#define CMIP_EVR     "CMIP_Evr"
#define ASN_METADATA "ASN_Metadata"
#define OID_MAPPING  "OID_Mapping"
#define GET_ACTIONS  "Get_Actions"
#define GET_NM_BIND  "Get_NameBindings"

class CMIPObj: public Information {
private:
  char *agentAET, *efdObjInst, *rootDN;

  char* SendOffLineRequest(char* type, char* val);
  void CMIPPerformSingleOperation(char* opName, int confirmed=0);
  Information** CMIPPerformScopedOperation(char* opName, int confirmed=0);
  
public:
  CMIPObj(char* aet="MIBCTL");
  ~CMIPObj();

  CMIPObj* clone();
  void SetAgentAET(char* aet);
  
  void SetObjectClass(char* val);
  char* GetObjectClass();
  
  void SetObjectInstance(char* val);
  char* GetObjectInstance();
  
  void CMIPCreateObject();

  void CMIPDeleteObject();
  Information** CMIPDeleteContainedInstances();

  void CMIPGetAttributes();
  Information** CMIPGetContainedInstances();

  void CMIPSetAttributes();
  Information** CMIPSetContainedInstances();

  void CMIPPerformAction(int confirmed=0);
  Information** CMIPPerformActionContainedInstances(int confirmed=0);

  Information* GetActions();
  Information* GetNameBindings();

  int NotificationsAvailable();
  void CreateEFD(char* systemInstance, char* filter);
  void DeleteEFD();
  Information* WaitForNotifications(int timeout);
  
  char* GetSyntaxInfo(char* syntax);
  char* ConvertOID(char* oid);  
};


/**********************************************************************************/

#define SNMP_GET         "SNMP_Get"
#define SNMP_GET_NEXT    "SNMP_GetNext"
#define SNMP_SET         "SNMP_Set"
#define SNMP_WALK        "SNMP_Walk"
#define SNMP_ATTR_INFO   "SNMP_Attr_Info"
#define SNMP_OID_MAPPING "SNMP_OID_Map"
#define SNMP_TRAP        "SNMP_Trap"

class SNMPObj: public Information {
 private:
   char* snmpAgtAddr;
   int snmpAgtPort, trapSockId;
	
   char* SendOffLineRequest(char* type, char* val) /* throw(char*) */ { return(proxy->SendOffLineRequest(type, NULL, val)); };
   void SNMPPerformSingleOperation(char* opName) /* throw(char*) */;

 public:
   SNMPObj();
   SNMPObj(char* remAgt, int remAgtPort=160);
   ~SNMPObj();

  SNMPObj* clone();
  void SetSnmpAgentAddress(char* remAgt, int remAgtPort=160);
  void SetCommunity(char* value);

  Information** SNMPWalk() /* throw(char*) */;	
  void SNMPGetAttribute() /* throw(char*) */     { RemoveAttribute("type"); SNMPPerformSingleOperation(SNMP_GET);      }
  void SNMPGetNextAttribute() /* throw(char*) */ { RemoveAttribute("type"); SNMPPerformSingleOperation(SNMP_GET_NEXT); }
  void SNMPSetAttribute() /* throw(char*) */     { SNMPPerformSingleOperation(SNMP_SET);                               }
  
  char* SNMPGetAttributeInfo(char* syntax) /* throw(char*) */ { return(SendOffLineRequest(SNMP_ATTR_INFO, syntax)); }
  char* ConvertOID(char* oid) /* throw(char*) */              { return(SendOffLineRequest(SNMP_OID_MAPPING, oid));  }

  int TrapsAvailable();
  void CreateTrapListener(int sockId);
  void DeleteTrapListener();
  Information* WaitForTraps(int timeout);
};



/**********************************************************************************/
 
#endif /* _PROXY_BINDINGS_H_ */
