/***********************************************************************

     IBM Confidential

     OCO Source Materials

     (C) Copyright IBM Corporation 1995-96

     The source code for this program is not published or
     otherwise divested of its trade secrets, irrespective of
     what has been deposited with the U.S. Copyright Office.

***********************************************************************/

#ifndef _PROXY_TYPES_H_
#define _PROXY_TYPES_H_

#include "ProxyBasicIncludes.h"

typedef enum { mimeHTML, mimeVRML , mimeURLenc } MIME_Types;

typedef struct Record {
  char* name;
  char* value;
};

class PrefEntry {
public:
  char* fieldName;
  char* fieldValue;

  PrefEntry();
  ~PrefEntry();
};

typedef enum {
  numeric_string,   /* num OID - num Value */
  mixed_string,     /* sym OID - num Value (compatibility mode) */
  symbolic_string   /* sym OID - sym Value */
  } StringAPIMode;

typedef enum {
  noS,
  startS, 
  moreS, 
  endS,
  stdPacket,
  unknownS
  } Symbols;

typedef enum {
  unknown_type,
  char_type,
  int_type,
  long_type,
  string_type,
  opaque_type
  } EventInfoType;

typedef enum {
  low_priority,
  medium_priority,
  high_priority
} EventPriority;

typedef void(*GenericProc)(long eventTime, void* accessRights);

typedef struct {
  EventInfoType eventInfoType;
  union {
    char    charInfo;
    int     intInfo;
    long    longInfo;
    char*   stringInfo;
    void*   opaqueInfo; /* General Purpose Data */
  } eventData;
} EventInfoData;


class EventInfo {
private:
  int eventType;
  EventPriority eventPriority; /* (Ignored) */
  GenericProc handlerProcedure; /* Optional procedure to call in front of special events */
  EventInfoData eventData;
  int   componentId;  /* Id of the component that has generated the event */
  long  eventTime;    /* When the event has been generated */
  long  expireTime;   /* When the event expires (i.e. has to be processed): -1 = immediately */
  void* accessRights; /* Access rights to be used when the event is processed (ignored) */

public:
  EventInfo(int eventType, int componentId, GenericProc handlerProcedure, long expireTime=-1);

  int GetEventType()            { return(eventType);            };
  EventPriority* GetPriority()  { return(&eventPriority);       };
  EventInfoData* GetEventData() { return(&eventData);           };
  GenericProc GetHandlerProcedure() { return(handlerProcedure); };
  void* GetAccessRights()       { return(accessRights);         };
  long GetActualTime()          { return(eventTime);            };
  void SetActualTime();
  long GetExpireTime()          { return(expireTime);            };
};


typedef void(*EvtHandlerProc)(EventInfo*);

typedef struct {
  EvtHandlerProc theEvtHandler;
  int            theComponentId; /* Component that implements the handler */
} EventHandlerEntry;

#define ACTUAL_CLASS_OID        "2.9.3.4.3.42"

#define OBJECT_CLASS_OID        "2.9.3.2.7.65"
#define OBJECT_CLASS_STR        "objectClass"

#define OBJECT_INST_OID         "2.9.3.2.7.61" /* managedObjectInstance */
#define OBJECT_INST_STR         "managedObjectInstance"

#define CURR_TIME_OID           "currentTime"  /* it does not exist     */
#define CURR_TIME_STR           "currentTime"

#define ACT_RSP_OID             ""
#define ACT_RSP_STR             "actionRsp"


#define NUM_ENTRIES             64

#endif /* _PROXY_TYPES_H_ */
