/************************************************************

  Name:         Proxy.java

  Description:  Proxy class implementation

  Coded by:     Luca Deri (lde@zurich.ibm.com)

  Licensed Materials - Property of IBM

  (C) Copyright IBM Corporation 1995-96.  All Rights Reserved.

**************************************************************/


import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class Proxy extends java.lang.Object {
  private ProxyThread proxyThread;
  public boolean asyncMode;
  public String result;
  public ProxyException exc;

  public Proxy(String host, int proxyPort) 
    throws UnknownHostException, IOException {
    proxyThread = new ProxyThread(this, host, proxyPort);
  }
	
  public void dispose() throws IOException{
    ;
  }

  public void SetAsynchMode(boolean mode) {
    asyncMode = mode;
  }

  public boolean OperationCompleted() {
    return(proxyThread.OperationCompleted());
  }

  public synchronized void WaitUntilCompletion() throws InterruptedException {
    proxyThread.completedSem.WaitUntilItUnlocks();
  }

  public void SetTimeoutForMgmtOperations(short timeout) { 
    proxyThread.SetTimeoutForMgmtOperations(timeout);
  }
  
  public short GetTimeoutForMgmtOperations() { 
    return(proxyThread.GetTimeoutForMgmtOperations()); 
  }
	  
  public Vector AsyncSendRequest(String operation, String context, 
				 Information input) throws ProxyException
    {
      Vector v = new Vector();
      
      exc = null;

      proxyThread.SetParameters(operation, context, input, null, 0, v, null);
      new Thread(proxyThread).start();
      return(v);
  }

  public Vector SendRequest(boolean asyncMode, String operation, 
			    String context, Information input) throws ProxyException
    {
      Vector v;

      SetAsynchMode(asyncMode);
      v = AsyncSendRequest(operation, context, input);

    if(asyncMode == false)
      {
	try {
	  WaitUntilCompletion();
	} catch(Exception e) {
	  e.printStackTrace();
	  v = null;
	}
      
	if(exc != null)
	  throw(exc);

	if(v.capacity() == 0)
	  v = null;

	// System.out.println(v);
      }     

      return(v);
  }

  public void AsyncSendOffLineRequest(String operation, String context,
				 String input) throws ProxyException
  {
    result = null;
    exc = null;
    proxyThread.SetParameters(operation, context, null, input, 1, null, null);
    new Thread(proxyThread).start();
  }
  
  public String SendOffLineRequest(boolean asyncMode, String operation, String context,
				   String input) throws ProxyException
  {
      SetAsynchMode(asyncMode);
      AsyncSendOffLineRequest(operation, context, input);

      if(asyncMode == false)
	{
	  try {
	    WaitUntilCompletion();
	  } catch(Exception e) {
	    result = null;
	  }

	  if(exc != null)
	    throw(exc);
	}
	
      return(result);
  }


  public Information AsyncSendOffLineReqWithMultipleOut(String operation, String context,
					String input) throws ProxyException
  {
    Information i = new Information();
    
    exc = null;

    proxyThread.SetParameters(operation, context, null, input, 2, null, i);
    new Thread(proxyThread).start();
    return(i);
  }
  
  public Information SendOffLineReqWithMultipleOut(boolean asyncMode, 
						   String operation, String context,
						   String input) throws ProxyException  {
    Information i;

    SetAsynchMode(asyncMode);
	  
    i = AsyncSendOffLineReqWithMultipleOut(operation, context, input);

    if(asyncMode == false)
      {
	try {
	  WaitUntilCompletion();
	} catch(Exception e) {
	  i = null;
	}

	if(exc != null)
	  throw(exc);	
      }

      return(i);
  }


  public int GetProxyBindingsVersion() throws ProxyException {
    return(proxyThread.GetProxyBindingsVersion());
  }
}
