import java.awt.*;
import java.lang.*;
import java.util.*;

class StatsGraph extends Panel 
{
   Font        font = null;
   String      title = null;
   int         maximum = 100;
   int         minimum = 0;
   int width, height;
   int	   xPlot=1;
   int     	   inset = 35;
   int 	   last_y=0;
   boolean     done=false;
   Graphics	   offGraphics;
   Image	   gfxBuff;
   Color       colors[] = { Color.blue, Color.red, Color.green, 
                            Color.orange, Color.cyan, Color.magenta };
   
   public final Color BACKGROUND = new Color(225, 225, 225);
   public final int   VERTICALLY = 0;
   public final int   HORIZONTALLY = 1;
   public final int   LEFT = 0;
   public final int   BELOW = 1;
   boolean scatterMode=false;

   public StatsGraph()
   {
      font = new Font("Helvetica", Font.PLAIN, 10);
      title = "misa.zurich.ibm.com: HTTP Traffic";   
      setBackground(Color.white);
   }

   public void setTitle(String theTitle)
   {
      title = theTitle;
   }


   public void setMin(int val) { minimum = val; }
   public void setMax(int val) { maximum = val; }

 public void addPoint(int val) 
   {
	int 	  extent;

	//System.out.println(val);

	if(val > maximum)
		val = maximum;
	else if(val < minimum)
		val = minimum;

	extent = (height*(val-minimum))/(maximum-minimum);
	offGraphics.setColor(Color.blue);
	if(xPlot == width)
	{
		offGraphics.copyArea(2, 1, width-2, height-1, -1, 0);
		if(scatterMode)
		{
			offGraphics.setColor(BACKGROUND);
			offGraphics.drawLine(width-1, 1, width-1, height);
			offGraphics.setColor(Color.blue);
			offGraphics.drawLine(width-1, last_y, width-1, height-extent);		
			last_y=height-extent;
		}
		else
		{
			offGraphics.drawLine(width-1, height-1, width-1, height-extent);
			offGraphics.setColor(BACKGROUND);
			offGraphics.drawLine(width-1, 1, width-1, height-extent);
		}
	}
	else
	{
		if(scatterMode)
		{
			offGraphics.drawLine(xPlot, last_y, xPlot, height-extent);
			last_y=height-extent;
		}
		else
			offGraphics.drawLine(xPlot, height-1, xPlot, height-extent);
		xPlot++;
	} 

	getGraphics().drawImage(gfxBuff, 0, 0, null);
  }

 public void paint(Graphics g) 
   {
   	  if(gfxBuff == null)
	  {	  
	    Dimension d=getSize();
	      width = d.width - 3 * inset;
 	      height = d.height - 2 * inset;

		  gfxBuff = createImage(d.width, d.height);
		  offGraphics = gfxBuff.getGraphics();
		
  	      // Put an outer border on the entire graph area.
	      offGraphics.setColor(Color.gray);
	      offGraphics.draw3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
	
	      // Fill the paint area, leaving a border.
	      offGraphics.setColor(BACKGROUND);
	      offGraphics.translate(inset, inset);
	      offGraphics.fillRect(0, 0, width, height);
	      
	      // Draw X and Y axes.
	      offGraphics.setColor(Color.black);
	      offGraphics.drawLine(0, 0, 0, height);
	      offGraphics.drawLine(0, height, width, height);
	
	      // Draw shading
	      offGraphics.setColor(Color.gray);
	      offGraphics.drawLine(0, 0, width, 0);
	      offGraphics.drawLine(width, 0, width, height);
	
	      drawTitle(offGraphics, width, height);
	      drawLegendMarker(offGraphics, width, height);
		drawTicksAndLabels(offGraphics, width, height);
	}
	
	g.drawImage(gfxBuff, 0, 0, null);
   }

   private void drawTitle(Graphics g, int width, int height) 
   {
      Font titleFont = new Font("Helvetica", Font.PLAIN, 14);
      FontMetrics metrics = getFontMetrics(titleFont);
      char characters[] = title.toCharArray();
      int theLength = title.length();
      Color  save = g.getColor();

      int newx = (width - metrics.charsWidth(characters, 0, theLength)) / 2;
      int newy = -15;

      g.setFont(titleFont);
      g.setColor(Color.black);
      g.drawChars(characters, 0, theLength, newx, newy);
      g.setColor(save);
   }

   private void drawLegendMarker(Graphics g, int width, int height)
   {
      Font legendFont = new Font("Helvetica", Font.ITALIC, 10);
      FontMetrics metrics = getFontMetrics(legendFont);

      legendFont = new Font("Helvetica", Font.PLAIN, 10);
      g.setFont(legendFont);

	g.setColor(Color.blue);
	g.drawLine(width+10,5, width+25,5);
	g.setColor(Color.black);
	g.drawString("Traffic", width+30,8);
   }

  private void drawTicksAndLabels(Graphics g, int width, int height)
   {
	g.setColor(Color.black);
	g.drawString("Time", width+5, height+5);
	g.drawString("Bytes", -30, -5);
	g.drawString("25 Kb", -30, 10);
	g.drawString("0", -10, height+5);
	g.setColor(Color.blue);
	g.drawString("Luca Deri, February 1996", width-50, height+30);
   }


}  
