/***********************************************************************

     IBM Corporation

     (C) Copyright IBM Corporation 1995-97

     Written by Luca Deri (lde@zurich.ibm.com)

***********************************************************************/

#include "ProxyBindings.h"

#define NUM_MANDATORY_ARGS  2

void help(char* progName)
{
  printf("Descr.:   Performs a CMIP M-ACTION on the specified instance\n");
  printf("Usage:    %s [-hLiaisonHost] [-pLiaisonPort] [-mmibName] [-cObjectClass]\n"
	 "          objectInstance actionId [-vactionValue]\n", progName);
  printf("Example:  %s \"netId=TelcoNet@systemId=(name atmNLOS_UK)@subNetworkId=(pString misaAtmSubnet_UK)\"\n"
	 "          releaseSNC \"(distinguishedName 'netId=TelcoNet@systemId=(name atmNLOS_UK)@subNetworkId=\n"
	 "          (pString misaAtmSubnet_UK)@atmSubNetworkConnectionId=SNC3')\"\n", progName);
  printf("Defaults: LiaisonHost=localhost, LiaisonPort=1998, mibName=MIBCTL\n");
  printf("Note:     actionValue may not be present (action dependent)\n");
}    


int main(int argc, char* argv[]) {
  char liaisonHost[64], mibName[32], objectInst[512];
  char actId[64], actVal[256], objectClass[64];
  int liaisonPort, idx, rc, option=0;
  CMIPObj *cmip;
  char **attrs;
  char **keys;

  if(argc < NUM_MANDATORY_ARGS+1)
    {
      help(argv[0]);
      return(-1);
    }
  else
    {
      /* Defaults */
      liaisonHost[0] = '\0', objectInst[0] = '\0', liaisonPort = 1998;
      actId[0] = '\0', actVal[0] = '\0', objectClass[0] = '\0';
      strcpy(mibName, "MIBCTL");

      for(idx=1; idx<argc; idx++)
	{
	  if(argv[idx][0] == '-') /* Optional argument */
	    {
	      switch(argv[idx][1])
		{
		case 'h':
		  strcpy(liaisonHost, &argv[idx][2]);
		  break;
		case 'p':
		  liaisonPort = atoi(&argv[idx][2]);
		  break;
		case 'm':
		  strcpy(mibName, &argv[idx][2]);
		  break;
		case 'v':
		  strcpy(actVal, &argv[idx][2]);
		  break;
		case 'c':
		  strcpy(objectClass, &argv[idx][2]);
		  break;
		default:
		  printf("Unknown option: %s. Ignored.\n", argv[idx]);
		}
	    }
	  else
	    {
	      switch(option)
		{
		case 0:
		  strcpy(objectInst, argv[idx]);
		  break;
		case 1:
		  strcpy(actId, argv[idx]);
		  break;
		default:
		  printf("Too many parameters: '%s' ignored.\n", argv[idx]);
		  break;
		}
	      option++;
	    }
	}
    }

  if(option < NUM_MANDATORY_ARGS)
    {
      printf("Error: Missing mandatory arguments.\n");
      help(argv[0]);
      return(-1);
    }
  
  try {
    cmip = new CMIPObj(mibName);
    cmip->UseProxy(liaisonHost, liaisonPort);

    if(objectClass[0] != '\0')
      cmip->SetObjectClass(objectClass);

    cmip->SetObjectInstance(objectInst);
    cmip->SetAttribute("actionType", actId);

    if(actVal[0] != '\0')
      cmip->SetAttribute("actionInfoArg", actVal);

    cmip->CMIPPerformAction(1 /* Confirmed - Put '0' for unconfirmed actions */); 

    attrs = cmip->GetAttributes();
    keys  = cmip->GetAttributeKeys();

    for(idx=0; keys[idx] != NULL; idx++)    
      printf("%s\t%s\n", keys[idx], attrs[idx]);

    rc = 0;
  } catch(char* str) {
    printf("Exception: %s\n", str);    
    rc = -1;
  }
  
  if(cmip != NULL)
    delete cmip;

  return(0);
}
