/***********************************************************************

     IBM Corporation

     (C) Copyright IBM Corporation 1995-97

     Written by Luca Deri (lde@zurich.ibm.com)

***********************************************************************/

#include "ProxyBindings.h"

#define NUM_MANDATORY_ARGS  1

void help(char* progName)
{
  printf("Descr.:   Deletes the specified instance (and the contained instances is scope is set)\n");
  printf("Usage:    %s [-hLiaisonHost] [-pLiaisonPort] [-mmibName] [-sscope] [-ffilter]"
	 " [-cObjectClass] objectInstance\n", progName);
  printf("Example:  %s \"netId=TelcoNet@systemId=(name TelcoSys)\"\n", progName);
  printf("Defaults: LiaisonHost=localhost, LiaisonPort=1998, mibName=MIBCTL,\n"
	 "          scope/filter absent, objectClass=actualClass\n");
}    

int main(int argc, char* argv[]) {
  char liaisonHost[64], mibName[32], objectInst[512];
  char scope[64], filter[256], objectClass[64];
  int liaisonPort, idx, rc, option=0, idx1;
  CMIPObj *cmip;
  Information **info;
  char **attrs;
  char **keys;

  if(argc < NUM_MANDATORY_ARGS+1)
    {
      help(argv[0]);
      return(-1);
    }
  else
    {
      /* Defaults */
      liaisonHost[0] = '\0', objectInst[0] = '\0', liaisonPort = 1998;
      scope[0] = '\0', filter[0] = '\0', objectClass[0] = '\0';
      strcpy(mibName, "MIBCTL");

      for(idx=1; idx<argc; idx++)
	{
	  if(argv[idx][0] == '-') /* Optional argument */
	    {
	      switch(argv[idx][1])
		{
		case 'h':
		  strcpy(liaisonHost, &argv[idx][2]);
		  break;
		case 'p':
		  liaisonPort = atoi(&argv[idx][2]);
		  break;
		case 'm':
		  strcpy(mibName, &argv[idx][2]);
		  break;
		case 's':
		  strcpy(scope, &argv[idx][2]);
		  break;
		case 'f':
		  strcpy(filter, &argv[idx][2]);
		  break;
		case 'c':
		  strcpy(objectClass, &argv[idx][2]);
		  break;
		default:
		  printf("Unknown option: %s. Ignored.\n", argv[idx]);
		}
	    }
	  else
	    {
	      switch(option)
		{
		case 0:
		  strcpy(objectInst, argv[idx]);
		  break;
		default:
		  printf("Too many parameters: '%s' ignored.\n", argv[idx]);
		  break;
		}
	      option++;
	    }
	}
    }

  if(option < NUM_MANDATORY_ARGS)
    {
      printf("Error: Missing mandatory arguments.\n");
      help(argv[0]);
      return(-1);
    }
  
  try {
    cmip = new CMIPObj(mibName);
    cmip->UseProxy(liaisonHost, liaisonPort);
 
    if(objectClass[0] != '\0')
      cmip->SetObjectClass(objectClass);

    cmip->SetObjectInstance(objectInst);

    if(scope[0] != '\0')     
      cmip->SetAttribute("scope", scope);

    if(filter[0] != '\0')     
      cmip->SetAttribute("filter", filter);

    if(scope[0] != '\0') 
      {
	info = cmip->CMIPDeleteContainedInstances(); 
	
	for(idx1=0; info[idx1] != NULL; idx1++)    
	  {
	    attrs = info[idx1]->GetAttributes();
	    keys  = info[idx1]->GetAttributeKeys();
	    
	    for(idx=0; keys[idx] != NULL; idx++)    
	      printf("%s\t%s\n", keys[idx], attrs[idx]);
	    
	    printf("\n"); /* Separator */
	  }
      }
    else
      {
	cmip->CMIPDeleteObject(); 

	attrs = cmip->GetAttributes();
	keys  = cmip->GetAttributeKeys();
	
	for(idx=0; keys[idx] != NULL; idx++)    
	  printf("%s\t%s\n", keys[idx], attrs[idx]);
	
	printf("\n"); /* Separator */
      }


    rc = 0;
  } catch(char* str) {
    printf("Exception: %s\n", str);    
    rc = -1;
  }
  
  if(cmip != NULL)
    delete cmip;

  return(0);
}
