/***********************************************************************

     IBM Corporation

     (C) Copyright IBM Corporation 1995-97

     Written by Luca Deri (lde@zurich.ibm.com)

***********************************************************************/

#include "ProxyBindings.h"

#define NUM_MANDATORY_ARGS  3

void help(char* progName)
{
  printf("Descr.:   Sets the specified attribute of the specified instance\n");
  printf("Usage:    %s [-hLiaisonHost] [-pLiaisonPort] [-mmibName] [-cObjectClass]\n"
         "          objectInstance OID value\n", progName);
  printf("Example:  %s \"netId=TelcoNet@systemId=(name TelcoSys)\" systemTitle \"(nothing NULL)\"\n", progName);
  printf("Defaults: LiaisonHost=localhost, LiaisonPort=1998, mibName=MIBCTL, objectClass=actualClass\n");
}    


int main(int argc, char* argv[]) {
  char liaisonHost[64], mibName[32], objectInst[512], objectClass[64];
  char attrId[64], attrVal[256];
  int liaisonPort, idx, rc, option=0;
  CMIPObj *cmip;
  char **attrs;
  char **keys;

  if(argc < NUM_MANDATORY_ARGS+1)
    {
      help(argv[0]);
      return(-1);
    }
  else
    {
      /* Defaults */
      liaisonHost[0] = '\0', objectInst[0] = '\0', 
      liaisonPort = 1998, objectClass[0] = '\0';
      attrId[0] = '\0', attrVal[0] = '\0';
      strcpy(mibName, "MIBCTL");

      for(idx=1; idx<argc; idx++)
	{
	  if(argv[idx][0] == '-') /* Optional argument */
	    {
	      switch(argv[idx][1])
		{
		case 'h':
		  strcpy(liaisonHost, &argv[idx][2]);
		  break;
		case 'p':
		  liaisonPort = atoi(&argv[idx][2]);
		  break;
		case 'm':
		  strcpy(mibName, &argv[idx][2]);
		  break;
		case 'c':
		  strcpy(objectClass, &argv[idx][2]);
		  break;
		default:
		  printf("Unknown option: %s. Ignored.\n", argv[idx]);
		}
	    }
	  else
	    {
	      switch(option)
		{
		case 0:
		  strcpy(objectInst, argv[idx]);
		  break;
		case 1:
		  strcpy(attrId, argv[idx]);
		  break;
		case 2:
		  strcpy(attrVal, argv[idx]);
		  break;
		default:
		  printf("Too many parameters: '%s' ignored.\n", argv[idx]);
		  break;
		}
	      option++;
	    }
	}
    }

  if(option < NUM_MANDATORY_ARGS)
    {
      printf("Error: Missing mandatory arguments.\n");
      help(argv[0]);
      return(-1);
    }
  
  try {
    cmip = new CMIPObj(mibName);
    cmip->UseProxy(liaisonHost, liaisonPort);

    if(objectClass[0] != '\0')
      cmip->SetObjectClass(objectClass);

    cmip->SetObjectInstance(objectInst);
    cmip->SetAttribute(attrId, attrVal);

    cmip->CMIPSetAttributes(); 

    attrs = cmip->GetAttributes();
    keys  = cmip->GetAttributeKeys();

    for(idx=0; keys[idx] != NULL; idx++)    
      printf("%s\t%s\n", keys[idx], attrs[idx]);


    rc = 0;
  } catch(char* str) {
    printf("Exception: %s\n", str);    
    rc = -1;
  }
  
  if(cmip != NULL)
    delete cmip;

  return(0);
}
