/***********************************************************************

     IBM Corporation

     (C) Copyright IBM Corporation 1995-97

     Written by Luca Deri (lde@zurich.ibm.com)

***********************************************************************/

#include "ProxyBindings.h"

#define NUM_MANDATORY_ARGS  1

void help(char* progName)
{
  printf("Descr.:   Returns all the variables contained in a specified MIB\n");
  printf("Usage:    %s [-hLiaisonHost] [-pLiaisonPort] SNMPAgentHost [-sSNMPport] [-ccommunity]\n", progName);
  printf("Example:  %s kae.zurich.ibm.com\n", progName);
  printf("Defaults: LiaisonHost=localhost, LiaisonPort=1998, SNMPport=161, community=public\n");
}    

int main(int argc, char* argv[]) {
  char liaisonHost[64], snmpAgentHost[64], community[32], oid[128];
  int liaisonPort, snmpPort, idx, rc, option=0, idx1;
  SNMPObj *snmp;
  Information **info;
  char **attrs;
  char **keys;

  if(argc < NUM_MANDATORY_ARGS+1)
    {
      help(argv[0]);
      return(-1);
    }
  else
    {
      /* Defaults */
      liaisonHost[0] = '\0', snmpAgentHost[0] = '\0', oid[0] = '\0';
      strcpy(community, "public");      
      liaisonPort = 1998, snmpPort=161;

      for(idx=1; idx<argc; idx++)
	{
	  if(argv[idx][0] == '-') /* Optional argument */
	    {
	      switch(argv[idx][1])
		{
		case 'h':
		  strcpy(liaisonHost, &argv[idx][2]);
		  break;
		case 'p':
		  liaisonPort = atoi(&argv[idx][2]);
		  break;
		case 's':
		  snmpPort = atoi(&argv[idx][2]);		  
		  break;
		case 'c':
		  strcpy(community, &argv[idx][2]);
		  break;
		default:
		  printf("Unknown option: %s. Ignored.\n", argv[idx]);
		}
	    }
	  else
	    {
	      switch(option)
		{
		case 0:
		  strcpy(snmpAgentHost, argv[idx]);
		  break;
		default:
		  printf("Too many parameters: '%s' ignored.\n", argv[idx]);
		  break;
		}
	      option++;
	    }
	}
    }

  if(option < NUM_MANDATORY_ARGS)
    {
      printf("Error: Missing mandatory arguments.\n");
      help(argv[0]);
      return(-1);
    }
  
  try {
    snmp = new SNMPObj(snmpAgentHost, snmpPort);
    snmp->UseProxy(liaisonHost, liaisonPort);
    snmp->SetCommunity(community);
    info = snmp->SNMPWalk(); 

    for(idx1=0; info[idx1] != NULL; idx1++)    
      {
	attrs = info[idx1]->GetAttributes();
	keys  = info[idx1]->GetAttributeKeys();

	for(idx=0; keys[idx] != NULL; idx++)    
	  printf("%s\t%s\n", keys[idx], attrs[idx]);

	printf("\n"); /* Separator */
      }

    rc = 0;
  } catch(char* str) {
    printf("Exception: %s\n", str);    
    rc = -1;
  }
  
  if(snmp != NULL)
    delete snmp;

  return(0);
}
