/************************************************************

  Name:         Information.java

  Description:  Imformation class implementation

  Coded by:     Luca Deri (lde@zurich.ibm.com)

  Licensed Materials - Property of IBM

  (C) Copyright IBM Corporation 1995-96.  All Rights Reserved.

**************************************************************/


import java.lang.*;
import java.util.* ;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.io.IOException;


public class Information extends Object implements Cloneable {
  Hashtable attributes; 
  Proxy proxy;
  short symbolicOID;
  short timeout;
  public boolean asyncMode;

  static final short NumericString  = 1;
  static final short MixedString    = 2;
  static final short SymbolicString = 3;
  
  public Information() { 
    attributes = new Hashtable();	
    proxy = null;
    symbolicOID = MixedString;
    timeout = 10;
    asyncMode = false;
  }

  public void SetAsynchMode(boolean mode) {
    asyncMode = mode;
  }

  public synchronized void WaitUntilCompletion() {
    if(asyncMode == true)
      {	
	try {
	  proxy.WaitUntilCompletion();
	} catch(Exception e) {
	  e.printStackTrace();
	}
      }
  }

  public Object clone() {
    try {
      return(super.clone()); 
    }
    catch(CloneNotSupportedException exception) {
      return(null); // This won't happen
    }
  }

  public boolean OperationCompleted() {
    if(proxy == null)
      return(true);
    else
      return(proxy.OperationCompleted());
  }

  public void UseProxy(String address) throws UnknownHostException, IOException {
    proxy = new Proxy(address, 1998);
  }

  public void UseProxy(String address, int port) throws UnknownHostException, IOException {
    proxy = new Proxy(address, port);
  }

  public void SetTimeoutForMgmtOperations(short timeout) { 
    proxy.SetTimeoutForMgmtOperations(timeout);
  }

  public void SetAttribute(String name, Object value) throws IllegalArgumentException {
    if(value instanceof String)
      attributes.put(name, value);
    else
      throw(new IllegalArgumentException("value must be a (sub)instance of String"));
  }
		
  public void ToggleOIDRepresentation(short value) throws IllegalArgumentException {
    if((value < NumericString) || (value >  SymbolicString))
      throw(new IllegalArgumentException("value out of range"));
    else
      symbolicOID = value;     
  }

  public Object GetAttribute(String name)       { return(attributes.get(name)); }
	
  public void RemoveAttribute(String name)    { attributes.remove(name); }
  public Enumeration GetAttributes()    	{ return(attributes.elements()); }
  public Enumeration GetAttributeKeys()    	{ return(attributes.keys()); }
  public void RemoveAllAttributes() 		{ attributes.clear();  }

  public void CopyValue(Information in) { 
    attributes.clear();
    attributes = in.attributes;
  }
		
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("[attributes="+attributes);
    buf.append(","+"Proxy="+proxy+"]");
    return buf.toString();
  }
	
  public String marshall() {

    SetAttribute("responseTimeout", String.valueOf(proxy.GetTimeoutForMgmtOperations()));

    Enumeration attrs = attributes.elements();
    Enumeration keys  = attributes.keys();
    StringBuffer buf = new StringBuffer();

    for(boolean first = true; attrs.hasMoreElements(); )
      {
	if(!first)
	  buf.append("&");
	buf.append(URLEncoder.encode((String)keys.nextElement())+"="+
		   URLEncoder.encode((String)attrs.nextElement()));
	first = false;		
      }

    RemoveAttribute("responseTimeout");
    return buf.toString();
  }

  public int GetProxyBindingsVersion() throws ProxyException {  
    if(proxy != null)
      return(proxy.GetProxyBindingsVersion());
    else
      throw(new ProxyException("Please specify what Proxy you wan to use (UseProxy method)"));
  }
}

