/************************************************************

  Name:         ProxyThread.java

  Description:  ProxyThread class implementation

  Coded by:     Luca Deri (lde@zurich.ibm.com)

  Licensed Materials - Property of IBM

  (C) Copyright IBM Corporation 1995-97.  All Rights Reserved.

**************************************************************/


import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class ProxyThread implements Runnable /* extends java.lang.Thread */ {
  private String remoteHost;
  private int port;
  private short timeout;
  static private String BINDINGS_VERSION = "Binding_Vers";
  private boolean running, opCompleted, notify=false;
  public Semaphore completedSem;
  Proxy proxy;
  String operation;
  String context; 
  Information input;
  String shortInput;
  int reqType;
  Vector v;
  Information i;

  public ProxyThread(Proxy _proxy, String host, int proxyPort)
    throws UnknownHostException, IOException {
    remoteHost = new String(host);
    port = proxyPort;
    running = false;
    completedSem = null;
    proxy = _proxy;
  }
	
  public void SetTimeoutForMgmtOperations(short _timeout) { 
    if(_timeout > 0)
      timeout=_timeout; 
  }
  
  public short GetTimeoutForMgmtOperations() { return(timeout); }
	
  public boolean GetThreadState() { 
    return(running); 
  }
  
  public boolean OperationCompleted() { 
    return(opCompleted); 
  }
  
  public void SetParameters(String _operation, String _context, 
			    Information _input, String _shortInput,
			    int _reqType, Vector _v,
			    Information _i) throws ProxyException {
    operation = _operation;
    context = _context;
    input = _input;
    shortInput = _shortInput;
    reqType = _reqType;
    v = _v;
    i = _i;
    opCompleted = false;
    completedSem = new Semaphore();
    completedSem.lock();
  }

  public void run() {
    
    running = true; //System.out.println("Start ["+reqType+"]"+this);
    
    try {
      switch(reqType) {
      case 0:
	this.SendRequest(v, operation, context, input);
	break;
      case 1:
	proxy.result = SendOffLineRequest(operation, context, shortInput);
	break;
      case 2:
	SendOffLineReqWithMultipleOut(i, operation, context, shortInput);
	break;
      }
      // proxy.exc = null;
    } catch(Exception e) {
      proxy.exc = (ProxyException)e;
    }

    running = false; // System.out.println("Stop "+this);
    opCompleted = true;
    completedSem.unlock();    
  }


  public Vector SendRequest(Vector v, String operation, String context, 
			    Information input) throws ProxyException 
  {
    boolean lastNullRsp=false, go=true;
    String str, oid=null, oidFormat, beginStr;
    Information info = null;
    URL url;
    InputStream is;
    
    try {
      if(input != null)
	beginStr = "&";
      else
	beginStr = "?";

      if(input.symbolicOID == Information.NumericString)
	oidFormat = "oidFormat=Numeric";
      else if(input.symbolicOID == Information.MixedString)
	oidFormat = "oidFormat=Mixed";
      else
	oidFormat = "oidFormat=Symbolic";

      str = "/BATCH/"+operation+"/"+context+"/"+input.marshall()+beginStr+oidFormat;

      //System.out.println("Sending: "+str);

      try {
	url = new URL("http", remoteHost, port, str);
	is = url.openStream();
      } catch(Exception e) {
	throw(new ProxyException("Unable to contact Liaison [is it up and running?]"));	
      }

      while(go)
	{
	  str = ReceiveString(is);
	  //System.out.println(str);
			
	  if(str == null)
	    {
	      if(lastNullRsp)
		go=false; 
	      else
		lastNullRsp=true; 

	      // Either: 1. the end 2. end for this message	
						      
	      if(info != null)  
		{
		  // if(v == null) v = new Vector();
		  v.addElement(info);
		  if(notify == true) v.notifyAll();
		  info=null;	
		}
	    }	
	  else
	    {
	      if(oid == null) 
		oid = str;
	      else
		{
		  if(info == null) info = new Information();
		  info.SetAttribute(oid, str);
		  oid=null;
		}
				
	      lastNullRsp=false;
	    }
	}

      if(info != null) 
	{
	  v.addElement(info);
	  if(notify == true) v.notifyAll();
	}

      str = ReceiveString(is);
	
      is.close();

      if(str != null)
	throw(new ProxyException(str));
      else
	return(v);
    } catch(Exception e) {
      throw(new ProxyException(e.getMessage()));	
    }	
  }
	

  public String SendOffLineRequest(String operation, String context, 
				   String input) throws ProxyException {
      String err;
      URL url;
      InputStream is;
      String str;

      if(input == null) 
	{
	  str = null;
	  throw(new ProxyException("Input Missing"));
	}

      try {
	if(context != null)  // Context
	  str = "/BATCH/"+operation+"/"+context+"/"+input;
	else
	  str = "/BATCH/"+operation+"/"+input;

	try {
	  url = new URL("http", remoteHost, port, str);
	  is = url.openStream();
	} catch(Exception e) {
	  str = null;
	  throw(new ProxyException("Unable to contact Liaison [is it up and running?]"));	
	}

	str = ReceiveString(is); // Response
	err = ReceiveString(is); // Error (if any)

	is.close();
      } catch(Exception e) {
	str = null;
	throw(new ProxyException(e.getMessage()));
      }

      if(err != null)
	{
	  str = null;
	  throw(new ProxyException(err));
	}
      else
	return(str);	
    }

  public Information SendOffLineReqWithMultipleOut(Information info,
						   String operation, 
						   String context, String input) throws ProxyException
    {
      boolean lastNullRsp=false, go=true;
      String str, oid=null;      
      URL url;
      InputStream is;

      if(input == null)
	{
	  info = null;
	  throw(new ProxyException("Input Missing"));
	}

      try {
	if(context != null)  // Context
	  str = "/BATCH/"+operation+"/"+context+"/"+input;
	else
	  str = "/BATCH/"+operation+"/"+input;

	try {
	  url = new URL("http", remoteHost, port, str);
	  is = url.openStream();
	} catch(Exception e) {
	  info = null;
	  throw(new ProxyException("Unable to contact Liaison [is it up and running?]"));	
	}

	while(go)
	  {
	    str = ReceiveString(is);
	    // System.out.println("[1] "+str);
	    
	    if(str == null)
	      {
		if(lastNullRsp)
		  go=false; 
		else
		  lastNullRsp=true; // Either: 1) the end 2) end for this message	
	      }	
	    else
	      {
		if(oid == null) 
		  oid = str;
		else
		  {
		    // if(info == null) info = new Information();
		    info.SetAttribute(oid, str);
		    if(notify == true) info.notifyAll();
		    oid=null;
		  }
		
		lastNullRsp=false;
	      }
	  }
	
	str = ReceiveString(is);
	// System.out.println("[2] "+str);

	is.close();
      } catch(Exception e) {
	info = null;
	throw(new ProxyException("Liaison problem [unable to receive the response]"));
      }
      
      if(str != null)
	{
	  info = null;
	  throw(new ProxyException(str));
	}
      else
	return(info);	
    }
		
  private String ReceiveString(InputStream is) {
    int receivedChar;
    boolean recChar=false;
    String recStr;
    StringBuffer tmp = new StringBuffer();
    
    try {
      while((receivedChar = is.read()) != -1)
	{
	  if(receivedChar == (int)'\n')
	    break;
	  else
	    {
	      tmp.append((char)receivedChar);	
	      recChar = true;
	    }
	}

      if(recChar)
	recStr = tmp.toString();
      else
	recStr = null;

      //System.out.println("Received: "+recStr);
      //Thread.sleep(1000);
      return(recStr);
    } catch(Exception e) { 
      System.out.println(e.getMessage());
      return(null);
    }
  }								

  public synchronized String toString() {
    return ("[remoteHost="+remoteHost+"]");
  }
  
  public int GetProxyBindingsVersion() throws ProxyException {
    int val;
    String s = SendOffLineRequest(BINDINGS_VERSION, null, "");
    
    try {
      val = Integer.parseInt(s);
    } catch(Exception e) { 
      val = -1;
    }
    
    return(val);
  }
  
}
